--[[ Overall Documentation: 
	These pretty much all require MiST.
	Some arguments have weird names to deconflict. These names aren't visible to the end user,
	so its NBD.
	
	Not a ton of documentation, as this was really just meant for me. The functions are basically
	mixtures of other (mostly MiST) functions.
	Notes are mostly to remind me how they work without having to read them.
	Recommend loading with 
		assert(loadfile("C:\\Users\\willt\\Saved Games\\DCS.openbeta\\Missions\\Scripts\\WrenchFunctions.lua"))()
	in case edits need to be made mid mission.
	this will have to be changed to "DO SCRIPT FILE" prior to release.
]]
--[[ Respawn
	Wrench.respawn will manually respawn a group, and they will go about their business.
	Call will look like
	Wrench.respawn(string_GroupName, boolean_Drone, Table_Zone, Bool_Message) i.e.
	Wrench.respawn('groupname',false,nil,false) or Wrench.respawn('groupname',true.{'zone1', 'zone2'},true)
	
	Auto Respawn 
	Auto calls wrench.respawn when group is destroyed.
	Arguments are the same as Wrench.respawn.
]]

local myLog = mist.Logger:new('Wrench functions')
if not Wrench then
	Wrench = {}
end

function Wrench.respawn(group, drone, zones, msg)
	local msg = {}
	msg.text = "Respawning "..group
	msg.displayTime = 30
	msg.msgFor = {coa = {'all'}}
	if msg then
		mist.message.add(msg)
	end
	if zones then
		mist.respawnInZone(group, zones, true, 0)
	else
		mist.respawnGroup(group, true)
	end
	if drone == true then
		local con = Group.getByName(group):getController()
		con:setOption(AI.Option.Air.id.ROE, AI.Option.Air.val.ROE.WEAPON_HOLD)
		con:setOption(AI.Option.Air.id.REACTION_ON_THREAT, AI.Option.Air.val.REACTION_ON_THREAT.EVADE_FIRE)
		con:setOption(AI.Option.Air.id.PROHIBIT_AA, true)
	end
end
function Wrench.autorespawn(group,drone,zones,msg)
	if not Group.getByName(group) then
		mist.scheduleFunction(Wrench.respawn, {group, drone, zones,msg}, timer.getTime() + 1, 10, timer.getTime() + timer.getTime() + 2)
	end
end
--[[ Random Activation
	Activates a group after random time. 
	Wrench.activater will automatically call Wrench.respawn.
	Call would be like
	Wrench.random_activate(groupname,min,max) i.e.
	Wrench.random_activate('bf109',0,120)
]]
function Wrench.random_activate(groupname,tmin,tmax,drone,zone,msg)
local group = Group.getByName(groupname)
local waittime = math.random(tmin,tmax)
myLog:msg(waittime)
if msg then
	trigger.action.outText(groupname .. 'will spawn in ' .. waittime .. ' seconds.', 5 , false)
end
mist.scheduleFunction(Wrench.respawn,{group,drone,zones,msg} , timer.getTime() + waittime , 10 ,timer.getTime() + waittime+1 )
end

--[[ In Table
	used by other functions.
]]
function Wrench.inTable(tbl, item)
    for key, value in pairs(tbl) do
        if value == item then return true
    end
end
end

--[[ Is Target Detected
	
	This is a big one...
	
	Used in the SU-33 AWACS Hunt Mission.
	Will set a flag if the units in a table of groups are detected by a unit for a specified amount of time.
	playergroups is a table of group names of playable aircraft prefaced with [g].
		i.e. {'[g]players #001','[g]players','[g]players #002'}
			look up mist.makeunittable for details on this.
		This must be passed as a table, with the {}.
		This will (in effect)be trimmed down by the function to only include units occupied by players,
		so pass all groups with PLAYER or CLIENT spot(s)
			
	awacs is the name of the AWACS unit. i.e. 'usawacs'
	mapzone is a table with the name of a zone which will encompass the whole map, used to make a table needed by the function.
		i.e. {'wholemap'}
		This must be passed as a table, with the {}.
	coa_enum
		0 = neutral
		1 = red
		2 = blue
	dettype is the type of detection to be used. Will be one of:
		VISUAL
		OPTIC
		RADAR
		IRST
		RWR
		DLINK
	flag_num is the flag to set to true (1) when the units are detected long enough. is a number, i.e. 100
	dettime is how long they must be detected for, in seconds. i.e. 60
		this will also count backwards, at 1/10 speed, when they are not detected. Will not go <1.
	mesg Boolean, whether or not to notify players they are being detected.
	
	The whole shebang might look something like:
	detectedseconds = 0
	Wrench.countdetectedseconds({'[g]players #001','[g]players','[g]players #002'},"usawacs",{'wholemap'},1,RADAR,100,60,true)
]]
function Wrench.countdetectedseconds(playergroups,awacs,mapzone,coa_enum,dettype,flag_num,dettime,mesg)
	stopflag = trigger.misc.getUserFlag(flag_num)
if stopflag == 0 then 
	playerplanes = mist.makeUnitTable(playergroups)
	playerplanes["processed"] = nil
	ewrUnit = Unit.getByName(awacs)
	ewrCtrl = Unit.getController(ewrUnit)
	activeunits = mist.getUnitsInZones(playerplanes ,mapzone)
	playerslist = coalition.getPlayers(coa_enum)
	numberofplayers = #playerslist
	if numberofplayers < 1 then
		numberofplayers = 1
	end
	DetectedTargets = {}
	for i=1,#activeunits do
		local visible = {}
		local detected = {}
		detected, visible = Controller.isTargetDetected(ewrUnit, activeunits[i] , dettype)
		if visible == true then
			detectedseconds = detectedseconds + (1/#activeunits)
		else
			if detectedseconds > 1 then 
				detectedseconds = detectedseconds + -0.1
			end
		end
	end
		if (detectedseconds) > dettime then
			trigger.action.setUserFlag(flag_num, 1 )
		end

	if mesg then
		trigger.action.outText(tostring(detectedseconds), 1 , true)
	end
myLog:msg('count ' .. detectedseconds)
end
end

function Wrench.escort(escortee, escort, roeswitch)
	trigger.action.removeMark(1)
	trigger.action.removeMark(2)	
	if Unit.getByName(escortee) == nil then 
		return true
	end
	local escortee = Unit.getByName(escortee)
	if Group.getByName(escort) == nil then 
		trigger.action.setUserFlag(1, true)
		return true
	end
	local escort = Group.getByName(escort)
	if escort:getUnit(1) == nil then 
		trigger.action.setUserFlag(1, true)
		return true
	end
	local escortleader = escort:getUnit(1)
	local escortcon = Unit.getController(escortleader)
	local escorteepos = Unit.getPosition(escortee).p
	local escorteespeed = mist.vec.mag(escortee:getVelocity())	
	local escortpos = Unit.getPosition(escortleader)
	trigger.action.markToAll(1, 'escortpos', escortpos.p , false, '')
	local escortwps = {}
	local escortdist = mist.utils.get2DDist(escortpos.p ,escorteepos)
	if escortdist > 2500*2 then
		escortwps[1] = mist.fixedWing.buildWP(escortpos.p , TurningPoint ,200 ,escorteepos.y ,'baro')
		escortwps[2] = mist.fixedWing.buildWP(escorteepos , TurningPoint ,200 ,escorteepos.y ,'baro')
	elseif escortdist > 1500 then
		escortwps[1] = mist.fixedWing.buildWP(escortpos.p , TurningPoint ,250 ,escorteepos.y ,'baro')
		escortwps[2] = mist.fixedWing.buildWP(escorteepos , TurningPoint ,escorteespeed + 5.14444 ,escorteepos.y ,'baro')
	elseif escortdist > 1000 then
		escortwps[1] = mist.fixedWing.buildWP(escortpos.p , TurningPoint ,250 ,escorteepos.y ,'baro')
		escortwps[2] = mist.fixedWing.buildWP(escorteepos , TurningPoint ,escorteespeed ,escorteepos.y ,'baro')
	end
	mist.goRoute(escort ,escortwps)
	if roeswitch then
		local detected , visible , lastTime , type , distance , lastPos , lastVel = escortcon:isTargetDetected(Group.getByName('players'):getUnit(1), 'VISUAL', 'RADAR', 'OPTIC', 'RWR')
		if visible then
		trigger.action.setUserFlag(1, true)
			trigger.action.outText("we've been spotted!", 10 , false)
		end
	end
end


--[[Popsmoke
	Must have the addcommand line which is commented out below in the ME, which will call the function.
	
	missionCommands.addCommandForCoalition(2,"Ready for Smoke",Smokemenu,function() wrench_popsmoke('New Vehicle Group') end, nil)
	
]]
function Wrench.popsmoke(inf)
	_G[inf..'group'] = Group.getByName(inf)
	local coa = Group.getCoalition(_G[inf..'group'])
	Smokemenu = missionCommands.addSubMenuForCoalition(coa, 'Pop Smoke')
	trigger.action.removeOtherCommandForCoalition(coa, "I see Green Smoke.")
	trigger.action.removeOtherCommandForCoalition(coa, "I see Red Smoke.")
	trigger.action.removeOtherCommandForCoalition(coa, "I see White Smoke.")
	trigger.action.removeOtherCommandForCoalition(coa, "I see Orange Smoke.")
	trigger.action.removeOtherCommandForCoalition(coa, "I see Blue Smoke.")
	trigger.action.removeOtherCommandForCoalition(coa, "I see two mark the same color!")
	local myLog = mist.Logger:new('popsmoke')
	local groupdata = mist.getCurrentGroupData(inf)
	local currentldr = groupdata.units[1]["unitName"]
	local ldrpos = Unit.getByName(currentldr):getPosition().p
	local numberrand = math.random(0,4)
	numberrand = math.random(0,4)
	numberrand = math.random(0,7)
	numberrand = math.random(0,20)
	numberrand = math.random(0,4)
	local numberrand1 = math.floor(numberrand*5)
	myLog:msg(numberrand)
	myLog:msg(numberrand1)
	trigger.action.smoke(ldrpos, numberrand1)
	_G[inf..'color'] = numberrand1
	trigger.action.outText('Mark is on the deck.',10,false)
	missionCommands.addCommandForCoalition(coa,"I see Green Smoke.",Smokemenu,function() wrench_seensmoke(inf,0) end, nil)
	missionCommands.addCommandForCoalition(coa,"I see Red Smoke.",Smokemenu,function() wrench_seensmoke(inf,1) end, nil)
	missionCommands.addCommandForCoalition(coa,"I see White Smoke.",Smokemenu,function() wrench_seensmoke(inf,2) end, nil)
	missionCommands.addCommandForCoalition(coa,"I see Orange Smoke.",Smokemenu,function() wrench_seensmoke(inf,3) end, nil)
	missionCommands.addCommandForCoalition(coa,"I see Blue Smoke.",Smokemenu,function() wrench_seensmoke(inf,4) end, nil)
	--missionCommands.addCommandForCoalition(2,"I see two mark the same color!",Smokemenu,function() wrench_seensmoke(inf,5) end, nil)
end
function Wrench.seensmoke(inf,seensmoke)
	if seensmoke == 5 then
		trigger.action.outText("Lets thy that again.",10,false)
		wrench_popsmoke(inf,enmy)
	end
	if seensmoke == _G[inf..'color'] then
		trigger.action.outText("Affirm, That's us.",10,false)
		trigger.action.removeOtherCommandForCoalition(coa, "I see Green Smoke.")
		trigger.action.removeOtherCommandForCoalition(coa, "I see Red Smoke.")
		trigger.action.removeOtherCommandForCoalition(coa, "I see White Smoke.")
		trigger.action.removeOtherCommandForCoalition(coa, "I see Orange Smoke.")
		trigger.action.removeOtherCommandForCoalition(coa, "I see Blue Smoke.")
		trigger.action.removeOtherCommandForCoalition(coa, "I see two mark the same color!")
	else
		trigger.action.outText("NEGATIVE, That's not us!",10,false)
	end
end